/** 
 *  Hyper Operating System V4 Advance
 *
 * @file  ipsnd_dt.c
 * @brief %jp{ǡ塼ؤ(ݡ 󥿥ƥ)}%en{Send to Data Queue}
 *
 * Copyright (C) 1998-2009 by Project HOS
 * http://sourceforge.jp/projects/hos/
 */



#include "core/core.h"
#include "object/dtqobj.h"



#if _KERNEL_SPT_IPSND_DTQ

#if _KERNEL_SPT_DPC

static void _kernel_dpc_psnd_dtq(ID dtqid, VP_INT data);

/** %jp{ǡ塼ؤ}%en{Send to Data Queue}
 * @param  dtqid    %jp{оݤΥǡ塼IDֹ}%en{ID number of the data queue to which the data element is sent}
 * @param  data     %jp{ǡ塼ǡ}%en{Data element tobe sent}
 * @retval E_OK     %jp{ｪλ}%en{Normal completion}
 * @retval E_ID     %jp{IDֹ(dtqid뤤ϻѤǤʤ)}%en{Invalid ID number(dtqid is invalid or unusable)}
 * @retval E_NOMEM  %jp{ٱ¹ѤΥ塼󥰥­}%en{Insufficient memory to store a service call for delayed execution}
 */
ER ipsnd_dtq(ID dtqid, VP_INT data)
{
	/* %jp{ID Υå} */
#if _KERNEL_SPT_IPSND_DTQ_E_ID
	if ( !_KERNEL_DTQ_CHECK_DTQID(dtqid) )
	{
		return E_ID;	/* %jp{IDֹ}%en{Invalid ID number} */
	}
#endif
	
	return _KERNEL_SYS_REQ_DPC(_kernel_dpc_psnd_dtq, dtqid, data);
}


void _kernel_dpc_psnd_dtq(ID dtqid, VP_INT data)
{
	_KERNEL_T_DTQCB  *dtqcb;
	_KERNEL_T_TSKHDL tskhdl;
	_KERNEL_T_TCB    *tcb;
	

	/* %jp{֥¸ߥå} */
#if _KERNEL_SPT_SND_DTQ_E_NOEXS
	if ( !_KERNEL_DTQ_CHECK_EXS(dtqid) )
	{
		_KERNEL_LEAVE_SVC();	/* %jp{ӥ뽪λ} */
		return;					/* %jp{֥̤} */
	}
#endif
	
	/* %jp{ǡ塼ȥ֥å} */
	dtqcb = _KERNEL_DTQ_ID2DTQCB(dtqid);
	
	/* %jp{ԤƬ饿Ф} */
	tskhdl = _KERNEL_DTQ_RMH_RQUE(dtqcb);
	if ( tskhdl != _KERNEL_TSKHDL_NULL )
	{
		VP_INT *p_data;

		/* %jp{ԤԤ} */
		tcb = _KERNEL_TSK_TSKHDL2TCB(tskhdl);		/* %jp{TCB} */
		_KERNEL_TSK_SET_ERCD(tcb, E_OK);			/* %jp{顼} */
		p_data = (VP_INT *)_KERNEL_TSK_GET_DATA(tcb);
		*p_data = data;
		_KERNEL_DSP_WUP_TSK(tskhdl);				/* %jp{Ԥ} */
		_KERNEL_DTQ_RMV_RTOQ(tskhdl);
	}
	else
	{
		const _KERNEL_T_DTQCB_RO *dtqcb_ro;
		_KERNEL_DTQ_T_DTQCNT     sdtqcnt;
		_KERNEL_DTQ_T_DTQCNT     dtqcnt;
		
		/* %jp{RO} */
		dtqcb_ro = _KERNEL_DTQ_GET_DTQCB_RO(dtqid, dtqcb);

		/* %jp{ǡ塼󥿼} */
		sdtqcnt = _KERNEL_DTQ_GET_SDTQCNT(dtqcb);
		dtqcnt  = _KERNEL_DTQ_GET_DTQCNT(dtqcb_ro);

		if ( sdtqcnt < dtqcnt )		/* %jp{塼˶Ϥ뤫} */
		{
			_KERNEL_DTQ_T_DTQCNT head;
			VP_INT               *dtq;

			/* %jp{ǡ塼} */
			head = _KERNEL_DTQ_GET_HEAD(dtqcb);
			dtq  = _KERNEL_DTQ_GET_DTQ(dtqcb_ro);

			/* %jp{ǡ塼ɲ} */
			if ( head < dtqcnt - sdtqcnt - 1 )
			{
				dtq[head + sdtqcnt] = data;
			}
			else
			{
				dtq[head + sdtqcnt - dtqcnt] = data;
			}
			
			/* %jp{ǡĿû} */
			sdtqcnt++;
			_KERNEL_DTQ_SET_SDTQCNT(dtqcb, sdtqcnt);
		}
	}
}


#else	/* _KERNEL_SPT_DPC */

/** %jp{ǡ塼ؤ(ݡ 󥿥ƥ)}%en{Send to Data Queue}
 * @param  dtqid    %jp{оݤΥǡ塼IDֹ}%en{ID number of the data queue to which the data element is sent}
 * @param  data     %jp{ǡ塼ǡ}%en{Data element tobe sent}
 * @retval E_OK     %jp{ｪλ}%en{Normal completion}
 * @retval E_ID     %jp{IDֹ(dtqid뤤ϻѤǤʤ)}%en{Invalid ID number(dtqid is invalid or unusable)}
 * @retval E_NOEXS  %jp{֥̤(оݥǡ塼̤Ͽ)}%en{Non-existant object(specified data queue is not registerd)}
 * @retval E_TMOUT  %jp{ݡ󥰼}%en{Polling failure}
 */
ER ipsnd_dtq(ID dtqid, VP_INT data)
{
	return psnd_dtq(dtqid, data);
}

#endif	/* _KERNEL_SPT_DPC */




#else	/* _KERNEL_SPT_IPSND_DTQ */


#if _KERNEL_SPT_SND_DTQ_E_NOSPT

/** %jp{ǡ塼ؤ}%en{Send to Data Queue}
 * @param  dtqid    %jp{оݤΥǡ塼IDֹ}%en{ID number of the data queue to which the data element is sent}
 * @param  data     %jp{ǡ塼ǡ}%en{Data element tobe sent}
 * @retval E_NOSPT  %jp{̤ݡȵǽ}%en{Unsupported function}
 */
ER ipsnd_dtq(ID dtqid, VP_INT data)
{
	return E_NOSPT;
}

#endif


#endif	/* _KERNEL_SPT_SIPSND_DTQ */



/* end of file */
