/** 
 *  Hyper Operating System V4 Advance
 *
 * @file  tget_mpf.c
 * @brief %jp{Ĺס񸻤γ(ॢȤ)}%en{Acquired Fixed-Sized Memory Block(with Timeout)}
 *
 * Copyright (C) 1998-2007 by Project HOS
 * http://sourceforge.jp/projects/hos/
 */



#include "core/core.h"
#include "object/mpfobj.h"



#if _KERNEL_SPT_TGET_MPF


#if _KERNEL_SPT_TGET_MPF && (_KERNEL_OPT_CODE_SIZE <= _KERNEL_OPT_SPEED)	/* %jp{tget_mpfǡͥʤ} */

/** %jp{Ĺסγ(ॢȤ)}%en{Acquired Fixed-Sized Memory Block(with Timeout)}
 * @param  mpfid	%jp{֥åоݤθĹסIDֹ}%en{ID number of the fixed-sized memory pool from which resource is acquired}
 * @param  p_blkf	%jp{Ĺס줿ѥåȤؤΥݥ}%en{Start address of the acquired memory block}
 * @retval E_OK     %jp{ｪλ}%en{Normal completion}
 * @retval E_ID     %jp{IDֹ(mpfid뤤ϻѤǤʤ)}%en{Invalid ID number(mpfid is invalid or unusable)}
 * @retval E_NOEXS  %jp{֥̤(оݸĹס뤬̤Ͽ)}%en{Non-existant object(specified fixed-sized memory pool is not registerd)}
 * @retval E_PAR    %jp{ѥ᡼顼(p_blk, tmout)}%en{Forced release from waiting(accept rel_wai while waiting)}
 * @retval E_RLWAI  %jp{Ԥ֤ζ(Ԥ֤δ֤rel_wai)}%en{Forced release from waiting(accept rel_wai while waiting)}
 * @retval E_DLT    %jp{Ԥ֥Ȥκ(Ԥ֤δ֤˸Ĺס뤬)}%en{Waiting object deleted(fixed-sized memory pool is deleted waiting)}
 */
ER tget_tmpf(ID mpfid, VP *p_blk, TMO tmout)
{
	_KERNEL_T_MPFCB_PTR  mpfcb;
	_KERNEL_T_TSKHDL     tskhdl;
	_KERNEL_T_TCB        *tcb;
	_KERNEL_MPF_T_BLKHDL blkhdl;
	VP                   blkptr;
	ER                   ercd;
	
	/* %jp{ƥȥå} */
#if _KERNEL_SPT_GET_MPF_E_CTX
	if ( _KERNEL_SYS_SNS_DPN() )
	{
		return E_CTX;			/* %jp{ƥȥ顼}%en{Context error} */
	}
#endif
	
	/* %jp{ID Υå} */
#if _KERNEL_SPT_GET_MPF_E_ID
	if ( !_KERNEL_MPF_CHECK_MPFID(mpfid) )
	{
		return E_ID;	/* %jp{IDֹ}%en{Invalid ID number} */
	}
#endif
	
	_KERNEL_ENTER_SVC();		/* %jp{ӥ}%en{enter service-call} */
	
	/* %jp{֥¸ߥå} */
#if _KERNEL_SPT_GET_MPF_E_NOEXS
	if ( !_KERNEL_MPF_CHECK_EXS(mpfid) )
	{
		_KERNEL_LEAVE_SVC();	/* %jp{ӥ뤫Ф}%en{leave service-call} */
		return E_NOEXS;			/* %jp{֥̤}%en{Non-existant object} */
	}
#endif

	/* %jp{ȥ֥å} */
	mpfcb = _KERNEL_MPF_ID2MPFCB(mpfid);
	
	/* %jp{Ĺס륫󥿼} */
	blkhdl = _KERNEL_MPF_GET_FREBLK(mpfcb);
	
	if ( blkhdl != _KERNEL_MPF_BLKHDL_NULL )
	{
		/* %jp{Ĺס񸻤Ǥ} */
		blkptr = _KERNEL_MPF_BLKHDL2PTR(mpfcb, blkhdl);
		*p_blk = blkptr;
		_KERNEL_MPF_SET_FREBLK(mpfcb, *(_KERNEL_MPF_T_BLKHDL *)blkptr);
		_KERNEL_MPF_SET_FBLKCNT(mpfcb, _KERNEL_MPF_GET_FBLKCNT(mpfhdl) - 1);
		ercd = E_OK;
	}
	else
	{
#if _KERNEL_SPT_TGET_MPF || _KERNEL_SPT_PGET_MPF
		if ( tmout == TMO_POL )
		{
			ercd = E_TMOUT;  /* %jp{ॢ}%en{Timeout} */
		}
		else
#endif
		{
				/* %jp{Ԥ֤ˤ} */
			tskhdl = _KERNEL_SYS_GET_RUNTSK();
			tcb = _KERNEL_TSK_TSKHDL2TCB(tskhdl);				/* %jp{TCB} */
			_KERNEL_TSK_SET_TSKWAIT(tcb, _KERNEL_TTW_MPF);
			_KERNEL_TSK_SET_WOBJID(tcb, mpfid);
			_KERNEL_TSK_SET_DATA(tcb, (VP_INT)p_blk);
			_KERNEL_DSP_WAI_TSK(tskhdl);
			_KERNEL_MPF_ADD_QUE(mpfcb, _KERNEL_MPF_GET_MPFCB_RO(mpfid, mpfcb), tskhdl);		/* %jp{Ԥɲ} */

#if _KERNEL_SPT_TGET_MPF
			if ( tmout != TMO_FEVR )
			{
				_KERNEL_SEM_ADD_TOQ(tskhdl, tmout);				/* %jp{ॢȥ塼ɲ} */
			}
#endif

			/* %jp{ǥѥåμ¹} */
			_KERNEL_DSP_TSK();

			/* %jp{顼ɤμ} */
			ercd = _KERNEL_TSK_GET_ERCD(tcb);
		}
	}
	
	_KERNEL_LEAVE_SVC();	/* %jp{֥̤}%en{Non-existant object} */
	
	return ercd;
}


#endif


#else	/* _KERNEL_SPT_PGET_MPF */


#if _KERNEL_SPT_PGET_MPF_E_NOSPT

/** %jp{Ĺסγ(ॢȤ)}%en{Acquired Fixed-Sized Memory Block(with Timeout)}
 * @param  mpfid	%jp{֥åоݤθĹסIDֹ}%en{ID number of the fixed-sized memory pool from which resource is acquired}
 * @param  p_blkf	%jp{Ĺס줿ѥåȤؤΥݥ}%en{Start address of the acquired memory block}
 * @retval E_NOSPT  %jp{̤ݡȵǽ}%en{Unsupported function}
 */
ER tget_mpf(ID mpfid)
{
	return E_NOSPT;
}

#endif


#endif	/* _KERNEL_SPT_PGET_MPF */



/* end of file */
