/** 
 *  Hyper Operating System V4 Advance
 *
 * @file  rel_wai.c
 * @brief %jp{Ԥ֤ζ}%en{Release Task from Waiting}
 *
 * Copyright (C) 1998-2009 by Project HOS
 * http://sourceforge.jp/projects/hos/
 */



#include "core/core.h"



#if _KERNEL_SPT_REL_WAI


/** %jp{Ԥ֤ζ}%en{Release Task from Waiting}
 * @param  tskid    %jp{Ԥ֤ζоݤΥIDֹ}%en{ID number of the task to be release task from waiting}
 * @retval E_OK     %jp{ｪλ}%en{Normal completion}
 * @retval E_ID     %jp{IDֹ(tskid뤤ϻѤǤʤ)}%en{Invalid ID number(tskid is invalid or unusable)}
 * @retval E_NOEXS  %jp{֥̤(оݥ̤Ͽ)}%en{Non-existant object(specified task is not registered)}
 * @retval E_OBJ    %jp{֥Ⱦ֥顼(оݥԤ֤Ǥʤ)}%en{Object state error(specified task is neither in WAITING state)}
 */
ER rel_wai(
		ID tskid)
{
	_KERNEL_T_TSKHDL tskhdl;
	_KERNEL_T_TCB    *tcb;

	if ( tskid == TSK_SELF )		/* %jp{Ѵ} */
	{
#if _KERNEL_SPT_REL_WAI_E_ID
		if ( _KERNEL_SYS_SNS_CTX() )
		{
			return E_ID;			/* %jp{IDֹ}%en{Invalid ID number} */
		}
#endif	
		/* %jp{¹楿} */
		tskhdl = _KERNEL_SYS_GET_RUNTSK();

		_KERNEL_ENTER_SVC();		/* %jp{ӥ}%en{enter service-call} */
	}
	else
	{
#if _KERNEL_SPT_REL_WAI_E_ID
		if ( !_KERNEL_TSK_CHECK_TSKID(tskid) )
		{
			return E_ID;			/* %jp{IDֹ}%en{Invalid ID number} */
		}
#endif
		
		_KERNEL_ENTER_SVC();		/* %jp{ӥ}%en%jp{enter service-call} */
		
		/* %jp{֥¸ߥå} */
#if _KERNEL_SPT_REL_WAI_E_NOEXS
		if ( !_KERNEL_TSK_CHECK_EXS(tskid) )
		{
			_KERNEL_LEAVE_SVC();	/* %jp{ӥФ}%en{%jp{leave service-call} */
			return E_NOEXS;			/* %jp{֥̤}%en{Non-existant object} */
		}
#endif
		/* %jp{IDֹѴ} */
		tskhdl = _KERNEL_TSK_ID2TSKHDL(tskid);
	}
	
	/* %jp{TCB}%en{get TCB} */
	tcb = _KERNEL_TSK_TSKHDL2TCB(tskhdl);
	
	/* %jp{ּå} */
#if _KERNEL_SPT_REL_WAI_E_OBJ
	{
		_KERNEL_TSK_T_TSKSTAT tskstat;

		tskstat = _KERNEL_TSK_GET_TSKSTAT(tcb);
		if ( !(tskstat & _KERNEL_TTS_WAI) )
		{
			_KERNEL_LEAVE_SVC();	/* %jp{ӥФ}%en{leave service-call} */
			return E_OBJ;			/* %jp{֥Ⱦ֥顼}%en{Object state error} */
		}
	}
#endif
	
	/* %jp{Ԥ} */
	_KERNEL_TRM_QUE(tskhdl);
	_KERNEL_TSK_SET_ERCD(tcb, E_RLWAI);		/* %jp{顼} */
	_KERNEL_DSP_WUP_TSK(tskhdl);			/* %jp{Ԥ} */
		
	/* %jp{ǥѥåμ¹}%en{Task dispatch} */
	_KERNEL_DSP_TSK();
	
	_KERNEL_LEAVE_SVC();		/* %jp{ӥФ}%en{leave service-call} */
	
	return E_OK;
}


#else	/* _KERNEL_SPT_REL_WAI */


#if _KERNEL_SPT_REL_WAI_E_NOSPT

/** %jp{Ԥ֤ζ}%en{Release Task from Waiting}
 * @param  tskid    %jp{Ԥ֤ζоݤΥIDֹ}%en{ID number of the task to be release task from waiting}
 * @retval E_NOSPT  %jp{̤ݡȵǽ}%en{Unsupported function}
 */
ER rel_wai(
		ID tskid)
{
	return E_NOSPT;
}

#endif


#endif	/* _KERNEL_SPT_REL_WAI */



/* end of file */
